#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

"""
File containing definitions for the capability execution routes of the Workspaces REST API
"""
import json
import logging

from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPPreconditionFailed
from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config

logger = logging.getLogger(__name__)


@view_config(route_name="create_version_from_previous_execution_script", renderer="json")
def create_version_from_previous_execution_script(request: Request) -> Response:
    """
    Pyramid view that accepts a request to create a capability version from the resubmit script
    of a previously execution, uses the workflow request id to lookup proper request
    URL: capability/request/execution/{wf_request_id}/create_version

    :param request: POST request, expecting JSON parameter "parameters"
    :return: 200 OK response with JSON-formatted info of newly created capability request
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability request with given id does not exist
    """

    wf_id = request.matchdict["wf_request_id"]
    params = json.loads(request.body.decode())

    execution = request.capability_info.lookup_execution_by_workflow_request_id(wf_id)

    if not execution:
        # CapabilityExecution with given current workflow id does not exist;
        # can't create a new version for it parent request
        does_not_exist_msg = f"Capability Execution not found for Workflow Request #{wf_id}. Cannot create new version"
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        parent_request = request.capability_info.lookup_capability_request(execution.capability_request_id)
        previous_params = parent_request.current_version.parameters

        parameters = {**previous_params, **params}

        new_capability_version = request.capability_info.create_capability_version(
            execution.capability_request_id, parameters=parameters
        )

        return new_capability_version.capability_request_id
