#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Tests for WorkflowRequestRestService """

# pylint: disable=E0401, W0621

import datetime
import http
import logging
from unittest.mock import MagicMock

import pytest
from pyramid.testing import DummyRequest
from workflow.server import WorkflowRequestRestService

from workspaces.workflow.schema import Workflow, WorkflowRequest
from workspaces.workflow.services.workflow_info import WorkflowInfo
from workspaces.workflow.services.workflow_service import WorkflowService

logger = logging.getLogger(__name__)


@pytest.fixture
def workflow() -> Workflow:
    """
    We'll need a null workflow.

    :return: minimally-initialized null workflow
    """
    wf = Workflow()
    wf.workflow_name = "null"
    wf.templates = []
    return wf


@pytest.fixture
def pyramid_request(mock_workflow_info: WorkflowInfo, mock_workflow_service: WorkflowService) -> DummyRequest:
    """
    We'll need a pyramid request.

    :param mock_workflow_info: mocked WorkflowInfo for creating request
    :param mock_workflow_service: mocked WorkflowService for creating request workflows

    :return: a pyramid request containing what we need for testing
    """
    request = DummyRequest()
    request.info = mock_workflow_info
    request.workflows = mock_workflow_service
    return request


@pytest.fixture
def workflow_request(pyramid_request: DummyRequest, workflow: Workflow) -> DummyRequest:
    """
    We'll need a workflow request.

    :param pyramid_request: initialized pyramid request using the supplied workflow
    :param workflow: workflow that will be the request's context

    :return: handy pyramid request to use in tests
    """
    pyramid_request.context = workflow
    pyramid_request.json_body = workflow.__json__()
    return pyramid_request


@pytest.fixture
def workflow_request_request(pyramid_request: DummyRequest) -> DummyRequest:
    """
    This is called a request_request because we're making a WorkflowRequest, and we're sticking it
    inside a Pyramid Request. Hence, WorkflowRequestRequest. This sucks, and we know it.

    :param pyramid_request: partially-initialized pyramid request
    :return: pyramid request containing a workflow request (the horror! the horror!)
    """
    wf_request = WorkflowRequest()
    wf_request.name = "null"
    wf_request.argument = '["-g"]'
    wf_request.state = "Unsubmitted"
    wf_request.files = []
    wf_request.results_dir = ""
    wf_request.created_at = datetime.datetime.now()
    wf_request.updated_at = datetime.datetime.now()
    wf_request.htcondor_job_id = -1
    pyramid_request.context = wf_request
    pyramid_request.json_body = wf_request.__json__()
    return pyramid_request


def test_create_workflow_request(workflow_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService can create a valid WorkflowRequest.

    :param workflow_request: pyramid request for use in making workflow request
    :return:
    """
    service = WorkflowRequestRestService(workflow_request)
    service.create_workflow_request()
    assert workflow_request.info.create_workflow_request.call_count == 1


def test_submit_workflow_request(workflow_request_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService can submit a pyramid request
    containing a valid WorkflowRequest

    :param workflow_request_request: mocked pyramid request
    for use in submitting workflow request

    :return:
    """
    service = WorkflowRequestRestService(workflow_request_request)
    service.submit_workflow()
    assert workflow_request_request.workflows.execute.call_count == 1


def test_create_and_submit_wf_request(workflow_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService can create and submit a WorkflowRequest
    in one swell foop.

    :param workflow_request: pyramid request
    :return:
    """
    service = WorkflowRequestRestService(workflow_request)
    service.create_and_submit()
    assert workflow_request.info.create_workflow_request.call_count == 1
    assert workflow_request.workflows.execute.call_count == 1


def test_announces_qa(workflow_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService announces QA Ready, QA Pass, or QA Fail as appropriate

    :param workflow_request:
    :return: pyramid request
    """

    # Init dummy request values
    matchdict_savepoint = workflow_request.matchdict
    body_savepoint = workflow_request.json_body
    workflow_request.matchdict["request_id"] = -1
    workflow_request.workflows.announce_qa = MagicMock()

    for msg_type in ["qa_ready", "qa_pass", "qa_fail"]:
        workflow_request.matchdict["msg_type"] = msg_type
        response = WorkflowRequestRestService(workflow_request).announce_qa()
        assert response.status_code == http.HTTPStatus.OK
        workflow_request.workflows.announce_qa.assert_called_with(-1, msg_type)

    # Reset dummy request properties to their initial values
    workflow_request_request.matchdict = matchdict_savepoint
    workflow_request_request.json_body = body_savepoint


def test_send_carta_url_to_aat(workflow_request_request: DummyRequest):
    # Init dummy request values
    matchdict_savepoint = workflow_request_request.matchdict
    body_savepoint = workflow_request_request.json_body
    workflow_request_request.matchdict["request_id"] = -1
    workflow_request_request.json_body = {"carta_url": "http://fake.carta/url"}

    # Mock out WorkflowService method call side effect
    workflow_request_request.workflows.send_carta_url_to_aat = MagicMock()
    response = WorkflowRequestRestService(workflow_request_request).send_carta_url_to_aat()

    workflow_request_request.workflows.send_carta_url_to_aat.assert_called_with(-1, "http://fake.carta/url")
    assert response.status_code == http.HTTPStatus.OK

    # Reset dummy request properties to their initial values
    workflow_request_request.matchdict = matchdict_savepoint
    workflow_request_request.json_body = body_savepoint


def test_send_do_not_calibate_msg_to_aat(workflow_request_request: DummyRequest):
    """
    Confirm that "Do Not Calibrate" message is sent to AAT with appropriate parameters

    :param workflow_request_request:
    :return:
    """
    # Init dummy request values
    matchdict_savepoint = workflow_request_request.matchdict
    body_savepoint = workflow_request_request.json_body

    request_id = -1
    workflow_request_request.matchdict["request_id"] = request_id
    sdm_id = "brain_000.58099.67095825232"
    workflow_request_request.json_body["sdm_id"] = sdm_id

    # Mock out WorkflowService method call side effect
    workflow_request_request.workflows.send_do_not_calibrate = MagicMock()
    expected_response = f"SUCCESS for request #{request_id}: Sent 'Do Not Calibrate' for {sdm_id} to AAT"
    try:
        response = WorkflowRequestRestService(workflow_request_request).send_do_not_calibrate()
        workflow_request_request.workflows.send_do_not_calibrate.assert_called_with(request_id, sdm_id)
        assert response.status_code == http.HTTPStatus.OK
        assert response.text == expected_response
    finally:
        # Reset dummy request properties to their initial values
        workflow_request_request.matchdict = matchdict_savepoint
        workflow_request_request.json_body = body_savepoint


def test_get_request_htcondor_id(workflow_request_request: DummyRequest):
    matchdict_savepoint = workflow_request_request.matchdict
    workflow_request_request.matchdict["request_id"] = -1

    workflow_request_request.info.lookup_workflow_request = MagicMock(return_value=workflow_request_request.context)
    response = WorkflowRequestRestService(workflow_request_request).get_request_htcondor_id()
    assert response.json_body["htcondor_job_id"] == "-1"

    # Reset dummy request properties to their initial values
    workflow_request_request.matchdict = matchdict_savepoint
